/*
Author: Mike Bok
Email: freaknigh@hotmail.com
Website: www.nighsoft.net
License and Rights: Full licensing and Rights available at www.nighsoft.net.
If licensing and rights not available at www.nighsoft.net, then all the following apply as bulleted:
-all rights reserved.
-you can not modify source code in this file.
-you can compile and use this source code.
*/

#include "main.h"

#define MAX_DATA_STORED 120000
#define MAX_BUF_SIZE 60000

int handle_client_hub(void *nothing);
int handle_client_server(void *nothing);
int handle_connect_hub(void *nothing);
int handle_connect_server(void *nothing);

void dejunk_bytes(char *buf, int amt);

struct client_info
{
	int c_id;
	struct sockaddr_in c_in;
	char recv_data[MAX_DATA_STORED];
        int recv_amt;
        int connected;
};

struct client_info client_hub;
struct client_info client_server;

void connect_to_hub()
{
	SDL_Thread *client_thread;
		
	if (login.visible)
	{
		draw_label(screen, "Connecting to main server...", &login.label, 255, 255, 255);
		SDL_Flip(screen);
	}
	else
	{
		strcpy(login.label.message, "Connecting to main server...");
	}

	client_thread = SDL_CreateThread(handle_connect_hub, NULL);
}

void connect_to_server(char *host_address)
{
	SDL_Thread *client_thread;
	
	if(str_match("127.0.0.1",host_address))
		client_thread = SDL_CreateThread(handle_connect_server, (void*)game.hub_address);
	else
		client_thread = SDL_CreateThread(handle_connect_server, (void*)host_address);
}

int handle_connect_hub(void *nothing)
{
	struct hostent * host;
	SDL_Thread *client_thread;

	#ifdef _M_IX86 //if windows
	WSADATA wsaData;
	
	WSAStartup(0x202,&wsaData);
	#endif
	
	if ((client_hub.c_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
	{
		if (login.visible)
		{
			draw_label(screen, "Socket creation failed.", &login.label, 255, 255, 255);
			SDL_Flip(screen);
		}
		else
		{
			strcpy(login.label.message, "Socket creation failed.");
		}
		return 0;
	}

	host = gethostbyname(game.hub_address);
 	memcpy((char*)&client_hub.c_in.sin_addr, (char*)host->h_addr, host->h_length);
 	client_hub.c_in.sin_family = AF_INET;
 	client_hub.c_in.sin_port = htons(4200);
 
 	if(connect(client_hub.c_id, (struct sockaddr *)&client_hub.c_in, sizeof client_hub.c_in) < 0)
	{
		if (login.visible)
		{
			draw_label(screen, "Connect failed, reconnecting...", &login.label, 255, 255, 255);
			SDL_Flip(screen);
		}
		else
		{
			strcpy(login.label.message, "Connect failed, reconnecting...");
		}
		
		if(connect(client_hub.c_id, (struct sockaddr *)&client_hub.c_in, sizeof client_hub.c_in) < 0)
		{
			if (login.visible)
			{
				draw_label(screen, "Reconnect failed.", &login.label, 255, 255, 255);
				SDL_Flip(screen);
			}
			else
			{
				strcpy(login.label.message, "Reconnect failed.");
			}
			return 0;
			
		}
	}
	
	
	client_thread = SDL_CreateThread(handle_client_hub, nothing);
	connect_hub_process();
	client_hub.connected = 1;
	return 0;
}

int handle_connect_server(void *message)
{
	struct hostent * host;
	SDL_Thread *client_thread;

	#ifdef _M_IX86 //if windows
	WSADATA wsaData;
	
	WSAStartup(0x202,&wsaData);
	#endif

	close_con_server();

	if ((client_server.c_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
	{
		if (server_select.visible) 
		{
			draw_label(screen, "Socket creation failed.", &server_select.label, 255, 255, 255);
			SDL_Flip(screen);
		}
		else
		{
			strcpy(server_select.label.message,"Socket creation failed.");
		}
		return 0;
	}

 	host = gethostbyname((char*)message);
 	memcpy((char*)&client_server.c_in.sin_addr, (char*)host->h_addr, host->h_length);
 	client_server.c_in.sin_family = AF_INET;
 	client_server.c_in.sin_port = htons(4600);
 
 	if(connect(client_server.c_id, (struct sockaddr *)&client_server.c_in, sizeof client_server.c_in) < 0)
	{
		if (server_select.visible) 
		{
			draw_label(screen, "Connect failed, reconnecting...", &server_select.label, 255, 255, 255);
			SDL_Flip(screen);
		}
		else
		{
			strcpy(server_select.label.message, "Connect failed, reconnecting...");
		}
		
		if(connect(client_server.c_id, (struct sockaddr *)&client_server.c_in, sizeof client_server.c_in) < 0)
		{
			if (server_select.visible) 
			{
				draw_label(screen, "Reconnect failed.", &server_select.label, 255, 255, 255);
				SDL_Flip(screen);
			}
			else
			{
				strcpy(server_select.label.message, "Reconnect failed.");
			}
			server_select.connecting = 0;
			return 0;
			
		}
	}
	
	client_thread = SDL_CreateThread(handle_client_server, NULL);
	connect_server_process();
	server_select.connecting = 0;
	server_select.connected = 1;
	client_server.connected = 1;
	return 0;
}

int handle_client_hub(void *nothing)
{
	char buf[MAX_BUF_SIZE + 1];
	int rcv_amt, i, temp_amt;

        client_hub.recv_amt = 0; //clear any left over data collected from a previous connection
        client_hub.recv_data[0] = '\0';
		
	while(1)  // if you rcv 0 data, the connection closed
	{
		buf[0] = '\0';
		
		rcv_amt = recv(client_hub.c_id, buf, MAX_BUF_SIZE, 0);
		
		if(rcv_amt <= 0) break;

		dejunk_bytes(buf, rcv_amt);
		
		if (rcv_amt + client_hub.recv_amt >= MAX_DATA_STORED) //if by adding more data you excede the space given, then trash the previous 29000 chars.
                {
                        client_hub.recv_amt = 0;
                        client_hub.recv_data[0] = '\0';
                }

		memcpy(client_hub.recv_data + client_hub.recv_amt, buf, rcv_amt); //add new data onto any previous data
                client_hub.recv_amt += rcv_amt;
		client_hub.recv_data[client_hub.recv_amt] = 0;

		temp_amt = 0;

		for(i=0;i<client_hub.recv_amt;) //run through collected data and check for processable data
                {
			int prev_amt;
			
			prev_amt = i;
			
                        split(buf,client_hub.recv_data,13,&i);
			
			if(client_hub.recv_data[i] == 0 && i < client_hub.recv_amt) i++;
			
                        if ((i >= client_hub.recv_amt) && (client_hub.recv_data[client_hub.recv_amt - 1] != 13)) //if you have reached the last segment and it doesnt have an end, then leave it to be processed later when it does
                        {
				temp_amt = client_hub.recv_amt - prev_amt;
                                memcpy(client_hub.recv_data,buf, temp_amt);
                        }
                        else if (buf[0] != '\0') //why process a blank string?
                        {
				process_packet_hub(buf); // found in proto.c
                        }
                }
		
		client_hub.recv_data[temp_amt] = '\0';
		client_hub.recv_amt = temp_amt;
		
	}
	
	//close(client_hub.c_id);
	
	//stuff to do since we lost the mothership
	disconnect_hub_process();
	
	client_hub.connected = 0;  // set avaible again
	
        return 0;
}

int handle_client_server(void *nothing)
{
	char buf[MAX_BUF_SIZE + 1];
	int rcv_amt, i, temp_amt;

        client_server.recv_amt = 0; //clear any left over data collected from a previous connection
        client_server.recv_data[0] = '\0';
		
	while(1)  // if you rcv 0 data, the connection closed
	{
		buf[0] = '\0';

		rcv_amt = recv(client_server.c_id, buf, MAX_BUF_SIZE, 0);
		
		if(rcv_amt <= 0) break;

		dejunk_bytes(buf, rcv_amt);

		if (rcv_amt + client_server.recv_amt >= MAX_DATA_STORED) //if by adding more data you excede the space given, then trash the previous 29000 chars.
                {
                        client_server.recv_amt = 0;
                        client_server.recv_data[0] = '\0';
                }

		memcpy(client_server.recv_data + client_server.recv_amt, buf, rcv_amt); //add new data onto any previous data
		client_server.recv_amt += rcv_amt;
		client_server.recv_data[client_server.recv_amt] = 0;

		temp_amt = 0;

		for(i=0;i<client_server.recv_amt;) //run through collected data and check for processable data
                {
			int prev_amt;
			
			prev_amt = i;
			
			split(buf,client_server.recv_data,13,&i);
			
			if(client_server.recv_data[i] == 0 && i < client_server.recv_amt) i++;
			
			if ((i >= client_server.recv_amt) && (client_server.recv_data[client_server.recv_amt - 1] != 13)) //if you have reached the last segment and it doesnt have an end, then leave it to be processed later when it does
			{
				
				temp_amt = client_server.recv_amt - prev_amt;
				memcpy(client_server.recv_data,buf, temp_amt);
			}
			else if (buf[0] != '\0') //why process a blank string?
			{
				process_packet_server(buf); // found in proto.c
			}

                }

		client_server.recv_data[temp_amt] = '\0';
		client_server.recv_amt = temp_amt;
	}
	
	//close(client_server.c_id);
	
	//whatever you want
	disconnect_server_process();
	
	client_server.connected = 0;  // set avaible again
	server_select.connected = 0;
	
        return 0;
}

void send_con_hub(char *message)
{
        char c[2] = {13,'\0'};
	printf("snt-h-%s\n",message);
	strcat(message,c);
        send(client_hub.c_id, message, strlen(message), 0);
}

void send_con_server(char *message)
{
        char c[2] = {13,'\0'};
	printf("snt-s-%s\n",message);
	strcat(message,c);
        send(client_server.c_id, message, strlen(message), 0);
}

void close_con_server()
{
     if(!client_server.connected) return;
     
#ifdef _WIN32
     closesocket(client_server.c_id);
#else
     close(client_server.c_id);
#endif
     
     //now wait until it is disconnected
     while(client_server.connected)
          SDL_Delay(100);
}

void close_con_hub()
{
	if(!client_hub.connected) return;
     
#ifdef _WIN32
	closesocket(client_hub.c_id);
#else
	shutdown(client_hub.c_id, SHUT_RDWR);
	close(client_hub.c_id);
#endif

	printf("now wait for hub disconnection\n");
	//now wait until it is disconnected
	while(client_hub.connected)
		SDL_Delay(100);
}

void dejunk_bytes(char *buf, int amt)
{
	int i;
	
	for(i=0;i<amt;i++)
	{
		int cur;
		
		cur = buf[i];
		
		if(cur == 13) continue;
		if(cur >= 32 && cur <= 127) continue;
		
		buf[i] = 0;
	}
}

